--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity P6SYNC is
	port (
		P6_HSYNCN	: in  std_logic;
		P6_VSYNCN	: in  std_logic;
		P6_RGBH		: in  std_logic_vector(3 downto 0);
		P6_CLK		: in  std_logic;
		CLK108M		: in  std_logic;
		RSTN		: in  std_logic;
		SRMODE		: out std_logic;
		CLKENB		: out std_logic;
		DT_HSYNCN	: out std_logic;
		DT_VSYNCN	: out std_logic;
		DT_RGBH		: out std_logic_vector(3 downto 0);
		SFTDT_U		: out std_logic_vector(2 downto 0);
		SFTDT_D		: out std_logic_vector(2 downto 0);
		SFTHS_U		: out std_logic_vector(2 downto 0);
		SFTHS_D		: out std_logic_vector(2 downto 0);
		LOCK1		: out std_logic;
		LOCK2		: out std_logic
	);
end P6SYNC;

architecture RTL of P6SYNC is

	component CLKSELROM is
		port (
			address	: in  std_logic_vector(6 downto 0);
			clock	: in  std_logic;
			q		: out std_logic_vector(2 downto 0)
		);
	end component;


	signal p6_clk_f		: std_logic_vector(29 downto 0);
	signal p6_hsyncn_f1	: std_logic;
	signal p6_hsyncn_f2	: std_logic;
	signal p6_hsyncn_f3	: std_logic;
	signal p6_hsyncn_f4	: std_logic;
	signal p6_hsyncn_f5	: std_logic;
	signal p6_hsyncn_f6	: std_logic;
	signal p6_hsyncn_f7	: std_logic;
	signal p6_vsyncn_f1	: std_logic;
	signal p6_vsyncn_f2	: std_logic;
	signal p6_vsyncn_f3	: std_logic;
	signal p6_rgbh_f1	: std_logic_vector(3 downto 0);
	signal p6_rgbh_f2	: std_logic_vector(3 downto 0);
	signal p6_rgbh_f3	: std_logic_vector(3 downto 0);
	signal p6_rgbh_f4	: std_logic_vector(3 downto 0);

	signal vcnt			: std_logic_vector(8 downto 0);

	signal clkr			: std_logic;
	signal clkr_f		: std_logic_vector(22 downto 0);

	signal clkr14m		: std_logic;
	signal clkr14m_f1	: std_logic_vector(6 downto 0);
	signal clkr14m_f2	: std_logic_vector(6 downto 0);
	signal clkr14m_f3	: std_logic_vector(6 downto 0);
	signal clkr14m_f4	: std_logic_vector(6 downto 0);
	signal clkr14m_f5	: std_logic_vector(6 downto 0);
	signal clkr14m_f6	: std_logic_vector(6 downto 0);
	signal clkenb_dt	: std_logic;
	signal clkenb_hs	: std_logic;

	signal tmg_enb		: std_logic;
	signal tmg_cal		: std_logic;
	signal tmg_cls		: std_logic;

	signal dtc			: std_logic;
	signal hsc			: std_logic;
	signal dtc_sum		: std_logic_vector(6 downto 0);
	signal hsc_sum		: std_logic_vector(6 downto 0);
	signal dtc_val		: std_logic_vector(2 downto 0);
	signal hsc_val		: std_logic_vector(2 downto 0);

	signal dt_rgbh_f1		: std_logic_vector(3 downto 0);
	signal dt_rgbh_f12		: std_logic_vector(3 downto 0);
	signal dt_rgbh_f13		: std_logic_vector(3 downto 0);
	signal dt_rgbh_f1_sel	: std_logic_vector(3 downto 0);
	signal dt_rgbh_f2		: std_logic_vector(3 downto 0);
	signal dt_rgbh_f22		: std_logic_vector(3 downto 0);
	signal dt_rgbh_f3		: std_logic_vector(3 downto 0);
	signal dt_rgbh_f32		: std_logic_vector(3 downto 0);
	signal dt_rgbh_f4		: std_logic_vector(3 downto 0);
	signal dt_rgbh_f42		: std_logic_vector(3 downto 0);
	signal dt_rgbh_f5		: std_logic_vector(3 downto 0);
	signal dt_rgbh_f52		: std_logic_vector(3 downto 0);
	signal dt_rgbh_f6		: std_logic_vector(3 downto 0);
	signal dt_hsyncn_f1		: std_logic;
	signal dt_hsyncn_f12	: std_logic;
	signal dt_hsyncn_f13	: std_logic;
	signal dt_hsyncn_f1_sel	: std_logic;
	signal dt_hsyncn_f2		: std_logic;
	signal dt_hsyncn_f22	: std_logic;
	signal dt_hsyncn_f3		: std_logic;
	signal dt_hsyncn_f32	: std_logic;
	signal dt_hsyncn_f4		: std_logic;
	signal dt_hsyncn_f42	: std_logic;
	signal dt_hsyncn_f5		: std_logic;
	signal dt_hsyncn_f52	: std_logic;
	signal dt_hsyncn_f6		: std_logic;

	signal dt_vsyncn_f1		: std_logic;

	signal srmode_i		: std_logic;
	signal clkdet_i		: std_logic;

	signal sftdt_u_i	: std_logic_vector(2 downto 0);
	signal sftdt_d_i	: std_logic_vector(2 downto 0);
	signal sfths_u_i	: std_logic_vector(2 downto 0);
	signal sfths_d_i	: std_logic_vector(2 downto 0);

	signal dt_lock1		: std_logic;
	signal dt_lock2		: std_logic;

begin

-- input latch
	process (CLK108M,RSTN)
	begin
		if (RSTN = '0') then
			p6_clk_f     <= (others => '0');
			p6_hsyncn_f1 <= '1';
			p6_hsyncn_f2 <= '1';
			p6_hsyncn_f3 <= '1';
			p6_hsyncn_f4 <= '1';
			p6_hsyncn_f5 <= '1';
			p6_hsyncn_f6 <= '1';
			p6_hsyncn_f7 <= '1';
			p6_vsyncn_f1 <= '1';
			p6_vsyncn_f2 <= '1';
			p6_vsyncn_f3 <= '1';
			p6_rgbh_f1   <= "0000";
			p6_rgbh_f2   <= "0000";
			p6_rgbh_f3   <= "0000";
			p6_rgbh_f4   <= "0000";
		elsif (CLK108M'event and CLK108M = '1') then
			p6_clk_f     <= p6_clk_f(28 downto 0) & P6_CLK;
			p6_hsyncn_f1 <= P6_HSYNCN;
			p6_hsyncn_f2 <= p6_hsyncn_f1;
			p6_hsyncn_f3 <= p6_hsyncn_f2;
			p6_hsyncn_f4 <= p6_hsyncn_f3;
			p6_hsyncn_f5 <= p6_hsyncn_f4;
			p6_hsyncn_f6 <= p6_hsyncn_f5;
			p6_hsyncn_f7 <= p6_hsyncn_f6;
			p6_vsyncn_f1 <= P6_VSYNCN;
			p6_vsyncn_f2 <= p6_vsyncn_f1;
			p6_vsyncn_f3 <= p6_vsyncn_f2;
			p6_rgbh_f1   <= P6_RGBH;
			p6_rgbh_f2   <= p6_rgbh_f1;
			p6_rgbh_f3   <= p6_rgbh_f2;
			p6_rgbh_f4   <= p6_rgbh_f3;
		end if;
	end process;

-- vartical counter
	process (CLK108M,RSTN)
	begin
		if (RSTN = '0') then
			vcnt <= (others => '0');
		elsif (CLK108M'event and CLK108M = '1') then
			if (clkdet_i = '0') then
				vcnt <= (others => '0');
			elsif (p6_hsyncn_f6 = '1' and p6_hsyncn_f7 = '0') then
				if (p6_vsyncn_f3 = '0') then
					vcnt <= (others => '0');
				else
					vcnt <= vcnt + 1;
				end if;
			end if;
		end if;
	end process;

	tmg_enb <= '1' when (vcnt >= 20 and vcnt < 250) else '0';
	tmg_cal <= '1' when (vcnt = 17 and p6_hsyncn_f6 = '1' and p6_hsyncn_f7 = '0') else '0';
	tmg_cls <= '1' when (vcnt = 18 and p6_hsyncn_f6 = '1' and p6_hsyncn_f7 = '0') else '0';


-- SR detect / clock detect
	process (CLK108M,RSTN)
	begin
		if (RSTN = '0') then
			srmode_i <= '0';
			clkdet_i <= '0';
		elsif (CLK108M'event and CLK108M = '1') then

			if (	p6_clk_f(3)  = '1' and
					p6_clk_f(7)  = '0' and
					p6_clk_f(10) = '1' and
					p6_clk_f(14) = '0' and
					p6_clk_f(18) = '1' and
					p6_clk_f(22) = '0' and
					p6_clk_f(25) = '1' and
					p6_clk_f(29) = '0'
				) then

				srmode_i <= '0';
				clkdet_i <= '1';

			elsif (	p6_clk_f(3)  = '1' and
					p6_clk_f(7)  = '1' and
					p6_clk_f(10) = '1' and
					p6_clk_f(14) = '1' and
					p6_clk_f(18) = '0' and
					p6_clk_f(22) = '0' and
					p6_clk_f(25) = '0' and
					p6_clk_f(29) = '0'
				) then

				srmode_i <= '1';
				clkdet_i <= '1';

			elsif (	p6_clk_f(29 downto 0) = "00" & X"0000000") then
				clkdet_i <= '0';
			elsif (	p6_clk_f(29 downto 0) = "11" & X"FFFFFFF") then
				clkdet_i <= '0';
			end if;

		end if;
	end process;


-- CLK generate

	clkr <= '1' when (p6_clk_f(1) = '1' and p6_clk_f(2) = '0') else '0';

	process (CLK108M,RSTN)
	begin
		if (RSTN = '0') then
			clkr_f <= (others => '0');
		elsif (CLK108M'event and CLK108M = '1') then
			if (srmode_i = '0') then
				clkr_f( 6 downto 0) <= clkr_f(5 downto 0) & clkr;
				clkr_f(22 downto 7) <= (others => '0');
			else
				clkr_f <= clkr_f(21 downto 0) & clkr;
			end if;
		end if;
	end process;

	clkr14m <= clkr_f(22) or clkr_f(15) or clkr_f(7) or clkr_f(0);


	process (CLK108M,RSTN)
	begin
		if (RSTN = '0') then
			clkr14m_f1 <= (others => '0');
			clkr14m_f2 <= (others => '0');
			clkr14m_f3 <= (others => '0');
			clkr14m_f4 <= (others => '0');
			clkr14m_f5 <= (others => '0');
			clkr14m_f6 <= (others => '0');
		elsif (CLK108M'event and CLK108M = '1') then
			clkr14m_f1 <= clkr14m_f1(5 downto 0) & clkr14m;
			clkr14m_f2 <= clkr14m_f2(5 downto 0) & clkr14m_f1(6);
			clkr14m_f3 <= clkr14m_f3(5 downto 0) & clkr14m_f2(6);
			clkr14m_f4 <= clkr14m_f4(5 downto 0) & clkr14m_f3(6);
			clkr14m_f5 <= clkr14m_f5(5 downto 0) & clkr14m_f4(6);
			clkr14m_f6 <= clkr14m_f6(5 downto 0) & clkr14m_f5(6);
		end if;
	end process;


-- data/hsync change detect
	process (CLK108M,RSTN)
	begin
		if (RSTN = '0') then
			dtc <= '0';
		elsif (CLK108M'event and CLK108M = '1') then
			if ( (p6_rgbh_f2 xor p6_rgbh_f3) /= "0000") then
				dtc <= '1';
			else
				dtc <= '0';
			end if;
		end if;
	end process;

	process (CLK108M,RSTN)
	begin
		if (RSTN = '0') then
			hsc <= '0';
		elsif (CLK108M'event and CLK108M = '1') then
			if (p6_hsyncn_f2 = '0' and p6_hsyncn_f3 = '1') then
				hsc <= '1';
			else
				hsc <= '0';
			end if;
		end if;
	end process;


	process (CLK108M,RSTN)
	begin
		if (RSTN = '0') then
			dtc_sum <= (others => '0');
		elsif (CLK108M'event and CLK108M = '1') then
			if (tmg_cls = '1') then
				dtc_sum <= (others => '0');
			elsif (tmg_enb = '1' and dtc = '1') then
				if    (clkr14m_f3(0) = '1') then
					dtc_sum(0) <= '1';
				elsif (clkr14m_f3(1) = '1') then
					dtc_sum(1) <= '1';
				elsif (clkr14m_f3(2) = '1') then
					dtc_sum(2) <= '1';
				elsif (clkr14m_f3(3) = '1') then
					dtc_sum(3) <= '1';
				elsif (clkr14m_f3(4) = '1') then
					dtc_sum(4) <= '1';
				elsif (clkr14m_f3(5) = '1') then
					dtc_sum(5) <= '1';
				elsif (clkr14m_f3(6) = '1') then
					dtc_sum(6) <= '1';
				end if;
			end if;
		end if;
	end process;

	process (CLK108M,RSTN)
	begin
		if (RSTN = '0') then
			hsc_sum <= (others => '0');
		elsif (CLK108M'event and CLK108M = '1') then
			if (tmg_cls = '1') then
				hsc_sum <= (others => '0');
			elsif (tmg_enb = '1' and hsc = '1') then
				if    (clkr14m_f3(0) = '1') then
					hsc_sum(0) <= '1';
				elsif (clkr14m_f3(1) = '1') then
					hsc_sum(1) <= '1';
				elsif (clkr14m_f3(2) = '1') then
					hsc_sum(2) <= '1';
				elsif (clkr14m_f3(3) = '1') then
					hsc_sum(3) <= '1';
				elsif (clkr14m_f3(4) = '1') then
					hsc_sum(4) <= '1';
				elsif (clkr14m_f3(5) = '1') then
					hsc_sum(5) <= '1';
				elsif (clkr14m_f3(6) = '1') then
					hsc_sum(6) <= '1';
				end if;
			end if;
		end if;
	end process;


	U_CLKSELROM1: CLKSELROM
	port map (
		address	=> dtc_sum,
		clock	=> CLK108M,
		q		=> dtc_val
	);

	U_CLKSELROM2: CLKSELROM
	port map (
		address	=> hsc_sum,
		clock	=> CLK108M,
		q		=> hsc_val
	);

-- data/hsync latch timing generate
	process (CLK108M,RSTN)
	begin
		if (RSTN = '0') then
			dt_lock1 <= '0';
		elsif (CLK108M'event and CLK108M = '1') then
			if (clkdet_i = '0') then
				dt_lock1 <= '0';
			elsif (vcnt = 251) then
				dt_lock1 <= '1';
			end if;
		end if;
	end process;

	process (CLK108M,RSTN)
	begin
		if (RSTN = '0') then
			dt_lock2 <= '0';
		elsif (CLK108M'event and CLK108M = '1') then
			if (clkdet_i = '0') then
				dt_lock2 <= '0';
			elsif (dt_lock1 = '0') then
				dt_lock2 <= '0';
			elsif (tmg_cal = '1') then
				if    (sfths_d_i = "001" and (hsc_val = "100" or hsc_val = "101")) then
					dt_lock2 <= '0';
				elsif (sfths_d_i = "010" and (hsc_val = "101" or hsc_val = "110")) then
					dt_lock2 <= '0';
				elsif (sfths_d_i = "011" and (hsc_val = "110" or hsc_val = "111")) then
					dt_lock2 <= '0';
				elsif (sfths_d_i = "100" and (hsc_val = "111" or hsc_val = "001")) then
					dt_lock2 <= '0';
				elsif (sfths_d_i = "101" and (hsc_val = "001" or hsc_val = "010")) then
					dt_lock2 <= '0';
				elsif (sfths_d_i = "110" and (hsc_val = "010" or hsc_val = "011")) then
					dt_lock2 <= '0';
				elsif (sfths_d_i = "111" and (hsc_val = "011" or hsc_val = "100")) then
					dt_lock2 <= '0';
				elsif (dtc_val /= "000" and hsc_val /= "000") then
					dt_lock2 <= '1';
				end if;
			end if;
		end if;
	end process;


	process (CLK108M,RSTN)
	begin
		if (RSTN = '0') then
			sftdt_u_i <= (others => '0');
			sftdt_d_i <= (others => '0');
		elsif (CLK108M'event and CLK108M = '1') then
			if (clkdet_i = '0') then
				sftdt_u_i <= (others => '0');
				sftdt_d_i <= (others => '0');
			elsif (tmg_cal = '1') then
				if    (dt_lock1 = '1' and dt_lock2 = '0') then
					sftdt_u_i <= "100";
					sftdt_d_i <= dtc_val;
				elsif (dt_lock1 = '1' and dt_lock2 = '1') then
					if (sftdt_d_i = dtc_val or dtc_val = "000") then
						null;
					else
						if    (sftdt_d_i = "001") then
							if (dtc_val = "010" or dtc_val = "011" or dtc_val = "100") then
								sftdt_d_i <= sftdt_d_i + 1;
							else
								sftdt_d_i <= "111";
								sftdt_u_i <= sftdt_u_i - 1;
							end if;
						elsif (sftdt_d_i = "010") then
							if (dtc_val = "011" or dtc_val = "100" or dtc_val = "101") then
								sftdt_d_i <= sftdt_d_i + 1;
							else
								sftdt_d_i <= sftdt_d_i - 1;
							end if;
						elsif (sftdt_d_i = "011") then
							if (dtc_val = "100" or dtc_val = "101" or dtc_val = "110") then
								sftdt_d_i <= sftdt_d_i + 1;
							else
								sftdt_d_i <= sftdt_d_i - 1;
							end if;
						elsif (sftdt_d_i = "100") then
							if (dtc_val = "101" or dtc_val = "110" or dtc_val = "111") then
								sftdt_d_i <= sftdt_d_i + 1;
							else
								sftdt_d_i <= sftdt_d_i - 1;
							end if;
						elsif (sftdt_d_i = "101") then
							if (dtc_val = "110" or dtc_val = "111" or dtc_val = "001") then
								sftdt_d_i <= sftdt_d_i + 1;
							else
								sftdt_d_i <= sftdt_d_i - 1;
							end if;
						elsif (sftdt_d_i = "110") then
							if (dtc_val = "111" or dtc_val = "001" or dtc_val = "010") then
								sftdt_d_i <= sftdt_d_i + 1;
							else
								sftdt_d_i <= sftdt_d_i - 1;
							end if;
						elsif (sftdt_d_i = "111") then
							if (dtc_val = "001" or dtc_val = "010" or dtc_val = "011") then
								sftdt_d_i <= "001";
								sftdt_u_i <= sftdt_u_i + 1;
							else
								sftdt_d_i <= sftdt_d_i - 1;
							end if;
						end if;
					end if;
				end if;
			end if;
		end if;
	end process;


	process (CLK108M,RSTN)
	begin
		if (RSTN = '0') then
			sfths_u_i <= (others => '0');
			sfths_d_i <= (others => '0');
		elsif (CLK108M'event and CLK108M = '1') then
			if (clkdet_i = '0') then
				sfths_u_i <= (others => '0');
				sfths_d_i <= (others => '0');
			elsif (tmg_cal = '1') then
				if    (dt_lock1 = '1' and dt_lock2 = '0') then
					sfths_u_i <= "100";
					sfths_d_i <= hsc_val;
				elsif (dt_lock1 = '1' and dt_lock2 = '1') then
					if (sfths_d_i = hsc_val or hsc_val = "000") then
						null;
					else
						if    (sfths_d_i = "001") then
							if (hsc_val = "010" or hsc_val = "011" or hsc_val = "100") then
								sfths_d_i <= sfths_d_i + 1;
							else
								sfths_d_i <= "111";
								sfths_u_i <= sfths_u_i - 1;
							end if;
						elsif (sfths_d_i = "010") then
							if (hsc_val = "011" or hsc_val = "100" or hsc_val = "101") then
								sfths_d_i <= sfths_d_i + 1;
							else
								sfths_d_i <= sfths_d_i - 1;
							end if;
						elsif (sfths_d_i = "011") then
							if (hsc_val = "100" or hsc_val = "101" or hsc_val = "110") then
								sfths_d_i <= sfths_d_i + 1;
							else
								sfths_d_i <= sfths_d_i - 1;
							end if;
						elsif (sfths_d_i = "100") then
							if (hsc_val = "101" or hsc_val = "110" or hsc_val = "111") then
								sfths_d_i <= sfths_d_i + 1;
							else
								sfths_d_i <= sfths_d_i - 1;
							end if;
						elsif (sfths_d_i = "101") then
							if (hsc_val = "110" or hsc_val = "111" or hsc_val = "001") then
								sfths_d_i <= sfths_d_i + 1;
							else
								sfths_d_i <= sfths_d_i - 1;
							end if;
						elsif (sfths_d_i = "110") then
							if (hsc_val = "111" or hsc_val = "001" or hsc_val = "010") then
								sfths_d_i <= sfths_d_i + 1;
							else
								sfths_d_i <= sfths_d_i - 1;
							end if;
						elsif (sfths_d_i = "111") then
							if (hsc_val = "001" or hsc_val = "010" or hsc_val = "011") then
								sfths_d_i <= "001";
								sfths_u_i <= sfths_u_i + 1;
							else
								sfths_d_i <= sfths_d_i - 1;
							end if;
						end if;
					end if;
				end if;
			end if;
		end if;
	end process;


	process (CLK108M,RSTN)
	begin
		if (RSTN = '0') then
			clkenb_dt <= '0';
		elsif (CLK108M'event and CLK108M = '1') then

			if    (sftdt_u_i = "010" and sftdt_d_i = "001") then
				clkenb_dt <= clkr14m_f1(0);
			elsif (sftdt_u_i = "010" and sftdt_d_i = "010") then
				clkenb_dt <= clkr14m_f1(1);
			elsif (sftdt_u_i = "010" and sftdt_d_i = "011") then
				clkenb_dt <= clkr14m_f1(2);
			elsif (sftdt_u_i = "010" and sftdt_d_i = "100") then
				clkenb_dt <= clkr14m_f1(3);
			elsif (sftdt_u_i = "010" and sftdt_d_i = "101") then
				clkenb_dt <= clkr14m_f1(4);
			elsif (sftdt_u_i = "010" and sftdt_d_i = "110") then
				clkenb_dt <= clkr14m_f1(5);
			elsif (sftdt_u_i = "010" and sftdt_d_i = "111") then
				clkenb_dt <= clkr14m_f1(6);

			elsif (sftdt_u_i = "011" and sftdt_d_i = "001") then
				clkenb_dt <= clkr14m_f2(0);
			elsif (sftdt_u_i = "011" and sftdt_d_i = "010") then
				clkenb_dt <= clkr14m_f2(1);
			elsif (sftdt_u_i = "011" and sftdt_d_i = "011") then
				clkenb_dt <= clkr14m_f2(2);
			elsif (sftdt_u_i = "011" and sftdt_d_i = "100") then
				clkenb_dt <= clkr14m_f2(3);
			elsif (sftdt_u_i = "011" and sftdt_d_i = "101") then
				clkenb_dt <= clkr14m_f2(4);
			elsif (sftdt_u_i = "011" and sftdt_d_i = "110") then
				clkenb_dt <= clkr14m_f2(5);
			elsif (sftdt_u_i = "011" and sftdt_d_i = "111") then
				clkenb_dt <= clkr14m_f2(6);

			elsif (sftdt_u_i = "100" and sftdt_d_i = "001") then
				clkenb_dt <= clkr14m_f3(0);
			elsif (sftdt_u_i = "100" and sftdt_d_i = "010") then
				clkenb_dt <= clkr14m_f3(1);
			elsif (sftdt_u_i = "100" and sftdt_d_i = "011") then
				clkenb_dt <= clkr14m_f3(2);
			elsif (sftdt_u_i = "100" and sftdt_d_i = "100") then
				clkenb_dt <= clkr14m_f3(3);
			elsif (sftdt_u_i = "100" and sftdt_d_i = "101") then
				clkenb_dt <= clkr14m_f3(4);
			elsif (sftdt_u_i = "100" and sftdt_d_i = "110") then
				clkenb_dt <= clkr14m_f3(5);
			elsif (sftdt_u_i = "100" and sftdt_d_i = "111") then
				clkenb_dt <= clkr14m_f3(6);

			elsif (sftdt_u_i = "101" and sftdt_d_i = "001") then
				clkenb_dt <= clkr14m_f4(0);
			elsif (sftdt_u_i = "101" and sftdt_d_i = "010") then
				clkenb_dt <= clkr14m_f4(1);
			elsif (sftdt_u_i = "101" and sftdt_d_i = "011") then
				clkenb_dt <= clkr14m_f4(2);
			elsif (sftdt_u_i = "101" and sftdt_d_i = "100") then
				clkenb_dt <= clkr14m_f4(3);
			elsif (sftdt_u_i = "101" and sftdt_d_i = "101") then
				clkenb_dt <= clkr14m_f4(4);
			elsif (sftdt_u_i = "101" and sftdt_d_i = "110") then
				clkenb_dt <= clkr14m_f4(5);
			elsif (sftdt_u_i = "101" and sftdt_d_i = "111") then
				clkenb_dt <= clkr14m_f4(6);

			elsif (sftdt_u_i = "110" and sftdt_d_i = "001") then
				clkenb_dt <= clkr14m_f5(0);
			elsif (sftdt_u_i = "110" and sftdt_d_i = "010") then
				clkenb_dt <= clkr14m_f5(1);
			elsif (sftdt_u_i = "110" and sftdt_d_i = "011") then
				clkenb_dt <= clkr14m_f5(2);
			elsif (sftdt_u_i = "110" and sftdt_d_i = "100") then
				clkenb_dt <= clkr14m_f5(3);
			elsif (sftdt_u_i = "110" and sftdt_d_i = "101") then
				clkenb_dt <= clkr14m_f5(4);
			elsif (sftdt_u_i = "110" and sftdt_d_i = "110") then
				clkenb_dt <= clkr14m_f5(5);
			elsif (sftdt_u_i = "110" and sftdt_d_i = "111") then
				clkenb_dt <= clkr14m_f5(6);
			end if;

		end if;
	end process;



	process (CLK108M,RSTN)
	begin
		if (RSTN = '0') then
			clkenb_hs <= '0';
		elsif (CLK108M'event and CLK108M = '1') then

			if    (sfths_u_i = "010" and sfths_d_i = "001") then
				clkenb_hs <= clkr14m_f1(0);
			elsif (sfths_u_i = "010" and sfths_d_i = "010") then
				clkenb_hs <= clkr14m_f1(1);
			elsif (sfths_u_i = "010" and sfths_d_i = "011") then
				clkenb_hs <= clkr14m_f1(2);
			elsif (sfths_u_i = "010" and sfths_d_i = "100") then
				clkenb_hs <= clkr14m_f1(3);
			elsif (sfths_u_i = "010" and sfths_d_i = "101") then
				clkenb_hs <= clkr14m_f1(4);
			elsif (sfths_u_i = "010" and sfths_d_i = "110") then
				clkenb_hs <= clkr14m_f1(5);
			elsif (sfths_u_i = "010" and sfths_d_i = "111") then
				clkenb_hs <= clkr14m_f1(6);

			elsif (sfths_u_i = "011" and sfths_d_i = "001") then
				clkenb_hs <= clkr14m_f2(0);
			elsif (sfths_u_i = "011" and sfths_d_i = "010") then
				clkenb_hs <= clkr14m_f2(1);
			elsif (sfths_u_i = "011" and sfths_d_i = "011") then
				clkenb_hs <= clkr14m_f2(2);
			elsif (sfths_u_i = "011" and sfths_d_i = "100") then
				clkenb_hs <= clkr14m_f2(3);
			elsif (sfths_u_i = "011" and sfths_d_i = "101") then
				clkenb_hs <= clkr14m_f2(4);
			elsif (sfths_u_i = "011" and sfths_d_i = "110") then
				clkenb_hs <= clkr14m_f2(5);
			elsif (sfths_u_i = "011" and sfths_d_i = "111") then
				clkenb_hs <= clkr14m_f2(6);

			elsif (sfths_u_i = "100" and sfths_d_i = "001") then
				clkenb_hs <= clkr14m_f3(0);
			elsif (sfths_u_i = "100" and sfths_d_i = "010") then
				clkenb_hs <= clkr14m_f3(1);
			elsif (sfths_u_i = "100" and sfths_d_i = "011") then
				clkenb_hs <= clkr14m_f3(2);
			elsif (sfths_u_i = "100" and sfths_d_i = "100") then
				clkenb_hs <= clkr14m_f3(3);
			elsif (sfths_u_i = "100" and sfths_d_i = "101") then
				clkenb_hs <= clkr14m_f3(4);
			elsif (sfths_u_i = "100" and sfths_d_i = "110") then
				clkenb_hs <= clkr14m_f3(5);
			elsif (sfths_u_i = "100" and sfths_d_i = "111") then
				clkenb_hs <= clkr14m_f3(6);

			elsif (sfths_u_i = "101" and sfths_d_i = "001") then
				clkenb_hs <= clkr14m_f4(0);
			elsif (sfths_u_i = "101" and sfths_d_i = "010") then
				clkenb_hs <= clkr14m_f4(1);
			elsif (sfths_u_i = "101" and sfths_d_i = "011") then
				clkenb_hs <= clkr14m_f4(2);
			elsif (sfths_u_i = "101" and sfths_d_i = "100") then
				clkenb_hs <= clkr14m_f4(3);
			elsif (sfths_u_i = "101" and sfths_d_i = "101") then
				clkenb_hs <= clkr14m_f4(4);
			elsif (sfths_u_i = "101" and sfths_d_i = "110") then
				clkenb_hs <= clkr14m_f4(5);
			elsif (sfths_u_i = "101" and sfths_d_i = "111") then
				clkenb_hs <= clkr14m_f4(6);

			elsif (sfths_u_i = "110" and sfths_d_i = "001") then
				clkenb_hs <= clkr14m_f5(0);
			elsif (sfths_u_i = "110" and sfths_d_i = "010") then
				clkenb_hs <= clkr14m_f5(1);
			elsif (sfths_u_i = "110" and sfths_d_i = "011") then
				clkenb_hs <= clkr14m_f5(2);
			elsif (sfths_u_i = "110" and sfths_d_i = "100") then
				clkenb_hs <= clkr14m_f5(3);
			elsif (sfths_u_i = "110" and sfths_d_i = "101") then
				clkenb_hs <= clkr14m_f5(4);
			elsif (sfths_u_i = "110" and sfths_d_i = "110") then
				clkenb_hs <= clkr14m_f5(5);
			elsif (sfths_u_i = "110" and sfths_d_i = "111") then
				clkenb_hs <= clkr14m_f5(6);
			end if;

		end if;
	end process;


-- data latch
	process (CLK108M,RSTN)
	begin
		if (RSTN = '0') then
			dt_rgbh_f1  <= (others => '0');
			dt_rgbh_f12 <= (others => '0');
			dt_rgbh_f13 <= (others => '0');
			dt_rgbh_f2  <= (others => '0');
			dt_rgbh_f22 <= (others => '0');
			dt_rgbh_f3  <= (others => '0');
			dt_rgbh_f32 <= (others => '0');
			dt_rgbh_f4  <= (others => '0');
			dt_rgbh_f42 <= (others => '0');
			dt_rgbh_f5  <= (others => '0');
			dt_rgbh_f52 <= (others => '0');
			dt_rgbh_f6  <= (others => '0');
		elsif (CLK108M'event and CLK108M = '1') then

			if (clkenb_dt = '1') then
				dt_rgbh_f1 <= p6_rgbh_f4;
			end if;

			dt_rgbh_f12 <= dt_rgbh_f1;
			dt_rgbh_f13 <= dt_rgbh_f12;


			if (clkr14m_f2(1) = '1') then
				if (sftdt_u_i = "010") then
					dt_rgbh_f2  <= dt_rgbh_f1_sel;
				else
					dt_rgbh_f2  <= dt_rgbh_f1_sel;
				end if;
			end if;

			if (clkr14m_f2(4) = '1') then
				dt_rgbh_f22  <= dt_rgbh_f2;
			end if;

			if (clkr14m_f3(1) = '1') then
				if (sftdt_u_i = "011") then
					dt_rgbh_f3  <= dt_rgbh_f1_sel;
				else
					dt_rgbh_f3  <= dt_rgbh_f22;
				end if;
			end if;

			if (clkr14m_f3(4) = '1') then
				dt_rgbh_f32  <= dt_rgbh_f3;
			end if;

			if (clkr14m_f4(1) = '1') then
				if (sftdt_u_i = "100") then
					dt_rgbh_f4  <= dt_rgbh_f1_sel;
				else
					dt_rgbh_f4  <= dt_rgbh_f32;
				end if;
			end if;

			if (clkr14m_f4(4) = '1') then
				dt_rgbh_f42  <= dt_rgbh_f4;
			end if;

			if (clkr14m_f5(1) = '1') then
				if (sftdt_u_i = "101") then
					dt_rgbh_f5  <= dt_rgbh_f1_sel;
				else
					dt_rgbh_f5  <= dt_rgbh_f42;
				end if;
			end if;

			if (clkr14m_f5(4) = '1') then
				dt_rgbh_f52  <= dt_rgbh_f5;
			end if;

			if (clkr14m_f6(1) = '1') then
				if (sftdt_u_i = "110") then
					dt_rgbh_f6  <= dt_rgbh_f1_sel;
				else
					dt_rgbh_f6  <= dt_rgbh_f52;
				end if;
			end if;

		end if;
	end process;

	dt_rgbh_f1_sel <=
		dt_rgbh_f13 when (sftdt_d_i = "001" or sftdt_d_i = "010") else
		dt_rgbh_f1;


	process (CLK108M,RSTN)
	begin
		if (RSTN = '0') then
			dt_hsyncn_f1  <= '1';
			dt_hsyncn_f12 <= '1';
			dt_hsyncn_f13 <= '1';
			dt_hsyncn_f2  <= '1';
			dt_hsyncn_f22 <= '1';
			dt_hsyncn_f3  <= '1';
			dt_hsyncn_f32 <= '1';
			dt_hsyncn_f4  <= '1';
			dt_hsyncn_f42 <= '1';
			dt_hsyncn_f5  <= '1';
			dt_hsyncn_f52 <= '1';
			dt_hsyncn_f6  <= '1';
		elsif (CLK108M'event and CLK108M = '1') then

			if (clkenb_hs = '1') then
				dt_hsyncn_f1 <= p6_hsyncn_f4;
			end if;

			dt_hsyncn_f12  <= dt_hsyncn_f1;
			dt_hsyncn_f13  <= dt_hsyncn_f12;

			if (clkr14m_f2(1) = '1') then
				if (sfths_u_i = "010") then
					dt_hsyncn_f2  <= dt_hsyncn_f1_sel;
				else
					dt_hsyncn_f2  <= dt_hsyncn_f1_sel;
				end if;
			end if;

			if (clkr14m_f2(4) = '1') then
				dt_hsyncn_f22  <= dt_hsyncn_f2;
			end if;

			if (clkr14m_f3(1) = '1') then
				if (sfths_u_i = "011") then
					dt_hsyncn_f3  <= dt_hsyncn_f1_sel;
				else
					dt_hsyncn_f3  <= dt_hsyncn_f22;
				end if;
			end if;

			if (clkr14m_f3(4) = '1') then
				dt_hsyncn_f32  <= dt_hsyncn_f3;
			end if;

			if (clkr14m_f4(1) = '1') then
				if (sfths_u_i = "100") then
					dt_hsyncn_f4  <= dt_hsyncn_f1_sel;
				else
					dt_hsyncn_f4  <= dt_hsyncn_f32;
				end if;
			end if;

			if (clkr14m_f4(4) = '1') then
				dt_hsyncn_f42  <= dt_hsyncn_f4;
			end if;

			if (clkr14m_f5(1) = '1') then
				if (sfths_u_i = "101") then
					dt_hsyncn_f5  <= dt_hsyncn_f1_sel;
				else
					dt_hsyncn_f5  <= dt_hsyncn_f42;
				end if;
			end if;

			if (clkr14m_f5(4) = '1') then
				dt_hsyncn_f52  <= dt_hsyncn_f5;
			end if;

			if (clkr14m_f6(1) = '1') then
				if (sfths_u_i = "110") then
					dt_hsyncn_f6  <= dt_hsyncn_f1_sel;
				else
					dt_hsyncn_f6  <= dt_hsyncn_f52;
				end if;
			end if;

		end if;
	end process;

	dt_hsyncn_f1_sel <=
		dt_hsyncn_f13 when (sfths_d_i = "001" or sfths_d_i = "010") else
		dt_hsyncn_f1;


	process (CLK108M,RSTN)
	begin
		if (RSTN = '0') then
			dt_vsyncn_f1 <= '1';
		elsif (CLK108M'event and CLK108M = '1') then
			dt_vsyncn_f1 <= p6_vsyncn_f3;
		end if;
	end process;


	SRMODE <= srmode_i;

	CLKENB    <= clkr14m_f6(1);
	DT_HSYNCN <= dt_hsyncn_f6;
	DT_VSYNCN <= dt_vsyncn_f1;
	DT_RGBH   <= dt_rgbh_f6;

	SFTDT_U <= sftdt_u_i;
	SFTDT_D <= sftdt_d_i;
	SFTHS_U <= sfths_u_i;
	SFTHS_D <= sfths_d_i;

	LOCK1 <= dt_lock1;
	LOCK2 <= dt_lock2;

end RTL;
